@@ -1,6 +1,6 @@
 
 package MIME::EncWords;
-use 5.005;
+use v5.8;
 
 =head1 NAME
 
@@ -11,8 +11,7 @@ MIME::EncWords - deal with RFC-1522 encoded words (improved)
 I<L<MIME::EncWords> is aimed to be another implimentation
 of L<MIME::Words> so that it will achive more exact conformance with
 MIME specifications.  Additionally, it contains some improvements.
-Following synopsis and descriptions are inherited from its inspirer,
-with description of improvements and clarifications added.>
+Following synopsis and descriptions are inherited from its inspirer.>
 
 Before reading further, you should see L<MIME::Tools> to make sure that
 you understand where this module fits into the grand scheme of things.
@@ -97,20 +96,10 @@ Exporter::export_ok_tags(qw(all));
 
 ### Other modules:
 use Carp;
+use Encode;
 use MIME::Base64;
 use MIME::Charset qw(:trans);
 
-my @ENCODE_SUBS = qw(FB_CROAK decode encode from_to is_utf8 resolve_alias);
-if (MIME::Charset::USE_ENCODE) {
-    eval "use ".MIME::Charset::USE_ENCODE." \@ENCODE_SUBS;";
-} else {
-    require MIME::Charset::_Compat;
-    for my $sub (@ENCODE_SUBS) {
-        no strict "refs";
-        *{$sub} = \&{"MIME::Charset::_Compat::$sub"};
-    }
-}
-
 #------------------------------
 #
 # Globals...
@@ -118,14 +107,12 @@ if (MIME::Charset::USE_ENCODE) {
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = '0.03.2';
+$VERSION = "0.01";
 
 ### Nonprintables (controls + x7F + 8bit):
 #my $NONPRINT = "\\x00-\\x1F\\x7F-\\xFF";
-my $PRINTABLE = "\\x21-\\x7E";
-my $NONPRINT = qr{[^$PRINTABLE]}; # Improvement: Unicode support.
-my $UNSAFE = qr{[^\x01-\x20$PRINTABLE]};
-my $WIDECHAR = qr{[^\x00-\xFF]};
+my $NONPRINT = qr{[^\x20-\x7E]}; # Improvement: Unicode support.
+my $UNSAFE = qr{[^\x01-\x7E]};
 
 ### Max line length:
 my $MAXLINELEN = 76;
@@ -198,7 +185,6 @@ and probably I<not> what you want, but if you know that all charsets
 in the ENCODED string are identical, it might be useful to you.
 (Before you use this, please see L<MIME::WordDecoder/unmime>,
 which is probably what you want.)
-B<Note>: See also "Charset" option below.
 
 In the event of a syntax error, $@ will be set to a description
 of the error, but parsing will continue as best as possible (so as to
@@ -209,22 +195,6 @@ Any arguments past the ENCODED string are taken to define a hash of options:
 
 =over 4
 
-=item Charset
-
-B<Improvement by this module>:
-Name of character set by which data elements in scalar context
-will be converted.
-If this option is specified as special value C<"_UNICODE_">,
-returned value will be Unicode string.
-
-When Unicode/multibyte support is disabled
-(see L<MIME::Charset/USE_ENCODE>),
-this option will not have any effects.
-
-B<Note>:
-This feature is still information-lossy, I<except> when C<"_UNICODE_"> is
-specified.
-
 =item Field
 
 Name of the mail field this string came from.  I<Currently ignored.>
@@ -233,13 +203,13 @@ Name of the mail field this string came from.  I<Currently ignored.>
 
 B<Improvement by this module>:
 Adjacent encoded-words with same charset will be concatenated
-to handle multibyte sequences safely.
+(if that charset is supported by Encode) to handle multibyte sequences
+safely.
 
 B<Change by this module>:
 Malformed base64 encoded-words will be kept encoded.
-In this case $@ will be set.
 
-B<Compatibility with MIME::Words>:
+B<NOTE>:
 Whitespaces surrounding unencoded data will not be stripped.
 
 =cut
@@ -247,14 +217,16 @@ Whitespaces surrounding unencoded data will not be stripped.
 sub decode_mimewords {
     my $encstr = shift;
     my %params = @_;
-    my $cset = $params{"Charset"};
     my @tokens;
     $@ = '';           ### error-return
 
+    ### Collapse boundaries between adjacent encoded words:
+    $encstr =~ s{(\?\=)\s*(\=\?)}{$1$2}gs;
+    pos($encstr) = 0;
+    ### print STDOUT "ENC = [", $encstr, "]\n";
+
     ### Decode:
     my ($word, $charset, $encoding, $enc, $dec);
-    my $spc = '';
-    pos($encstr) = 0;
     while (1) {
         last if (pos($encstr) >= length($encstr));
         my $pos = pos($encstr);               ### save it
@@ -265,10 +237,8 @@ sub decode_mimewords {
                              \?([bq])      #  "?" + encoding +
                              \?([^?]+)     #  "?" + data maybe with spcs +
                              \?=           #  "?="
-			     ([\r\n\t ]*)
                             }xgi) {
 	    ($word, $charset, $encoding, $enc) = ($&, $1, lc($2), $3);
-	    my $tspc = $4;
 	    if ($encoding eq 'q') {
 		$dec = _decode_Q($enc);
 	    } else {
@@ -276,19 +246,17 @@ sub decode_mimewords {
 	    }
 	    unless (defined $dec) {
 		$@ .= qq|Illegal sequence in "$word" (pos $pos)\n|;
-		push @tokens, [$spc.$word];
-		$spc = '';
+		push @tokens, [$word];
 		next;
 	    }
 
 	    if (scalar(@tokens) and
 		lc($charset) eq lc(${$tokens[-1]}[1]) and
-		resolve_alias($charset)) { # Concat words if possible.
+		Encode::resolve_alias($charset)) { # Concat words if possible.
 		${$tokens[-1]}[0] .= $dec;
 	    } else {
 		push @tokens, [$dec, $charset];
 	    }
-	    $spc = $tspc;
             next;
         }
 
@@ -297,79 +265,27 @@ sub decode_mimewords {
         pos($encstr) = $pos;               # reset the pointer.
         if ($encstr =~ m{\G=\?}xg) {
             $@ .= qq|unterminated "=?..?..?=" in "$encstr" (pos $pos)\n|;
-            push @tokens, [$spc.'=?'];
-	    $spc = '';
+            push @tokens, ['=?'];
             next;
         }
 
         ### Case 3: are we looking at ordinary text?
         pos($encstr) = $pos;               # reset the pointer.
         if ($encstr =~ m{\G                # from where we left off...
-                         (.*?              #   shortest possible string,
+                         ([\x00-\xFF]*?    #   shortest possible string,
                           \n*)             #   followed by 0 or more NLs,
                          (?=(\Z|=\?))      # terminated by "=?" or EOS
-                        }xgs) {
-            length($1) or croak "MIME::EncWords: internal logic err: empty token\n";
-            push @tokens, [$spc.$1];
-	    $spc = '';
+                        }xg) {
+            length($1) or die "MIME::EncWords: internal logic err: empty token\n";
+            push @tokens, [$1];
             next;
         }
 
         ### Case 4: bug!
-        croak "MIME::EncWords: unexpected case:\n($encstr) pos $pos\n\t".
+        die "MIME::EncWords: unexpected case:\n($encstr) pos $pos\n\t".
             "Please alert developer.\n";
     }
-    push @tokens, [$spc] if $spc;
-
-    return (wantarray ? @tokens : join('',map {
-	&_convert($_->[0], $_->[1], $cset)
-	} @tokens));
-}
-
-#------------------------------
-
-# _convert RAW, FROMCHARSET, TOCHARSET
-#     Private: used by encode_mimewords() to convert string by other charset
-#     or to decode to Unicode.
-#     When source charset is unknown and Unicode string is requested, at first
-#     try well-formed UTF-8 then fallback to ISO-8859-1 so that almost all
-#     non-ASCII bytes will be preserved.
-sub _convert($$$) {
-    my $s = shift;
-    my $charset = shift || "";
-    my $cset = shift;
-    return $s unless MIME::Charset::USE_ENCODE;
-    return $s unless $cset;
-    return $s if uc($charset) eq uc($cset);
-
-    my $preserveerr = $@;
-
-    my $converted = $s;
-    if (is_utf8($s) or $s =~ $WIDECHAR) {
-	if ($cset ne "_UNICODE_") {
-	    $converted = encode($cset, $converted);
-	}
-    } elsif ($cset eq "_UNICODE_") {
-	if (!resolve_alias($charset)) {
-	    if ($s =~ $UNSAFE) {
-		$@ = '';
-		eval {
-		    $converted = decode("UTF-8", $converted, FB_CROAK());
-		};
-		if ($@) {
-		    $converted = $s;
-		    $converted = decode("ISO-8859-1", $converted);
-		}
-	    }
-	} else {
-	    $converted = decode($charset, $converted);
-	}
-    } elsif (resolve_alias($charset)) {
-	from_to($converted, $charset, $cset);
-    }
-
-    $@ = $preserveerr;
-    return $converted;
+    return (wantarray ? @tokens : join('',map {$_->[0]} @tokens));
 }
 
 #------------------------------
@@ -434,11 +350,10 @@ of characters:
 Returns the encoded string.
 
 B<Improvement by this module>:
-RAW may be a Unicode string when Unicode/multibyte support is enabled
-(see L<MIME::Charset/USE_ENCODE>).
-Furthermore, RAW may be a reference to that returned
-by L<"decode_mimewords"> on array context.  In latter case "Charset"
-option (see below) will be overridden (see also notes below).
+RAW may be a Unicode string.
+Furthermore, RAW may be an arrayref which is reference to that returned
+by decode_mimewords on array context.  In latter case Charset option
+(see below) will be overridden.
 
 Any arguments past the RAW string are taken to define a hash of options:
 
@@ -452,21 +367,18 @@ a.k.a. "Latin-1".
 =item Detect7bit
 
 B<Improvement by this modlue>:
-When "Encoding" option (see below) is specified as C<"a"> and "Charset"
+When L<"Encoding"> (see below) is specified as C<"a"> and Charset
 option is unknown, try to detect 7-bit charset on given RAW string.
 Default is C<"YES">.
-When Unicode/multibyte support is disabled,
-this option will not have any effects
-(see L<MIME::Charset/USE_ENCODE>).
 
 =item Encoding
 
 The encoding to use, C<"q"> or C<"b">.  The default is C<"q">.
 B<Improvement by this module>:
 You may also specify ``special'' values: C<"a"> will automatically choose
-recommended encoding to use (with charset conversion if alternative
-charset is recommended: see L<MIME::Charset>);
-C<"s"> will choose shorter one of either C<"q"> or C<"b">.
+recommended encoding to use (with charset conversion, if alternative
+charset is recommended); C<"s"> will choose shorter one of either C<"q">
+or C<"b">.
 
 =item Field
 
@@ -477,27 +389,23 @@ encoded header.
 
 =back
 
-B<Notes on improvement by this module>:
-When RAW is an arrayref,
-adjacent encoded-words are concatenated.  Then they are splitted taking
-care of character boundaries of multibyte sequences, when Unicode/multibyte
-support is enabled.
-Portions for unencoded data should include surrounding whitespace(s), or
-they will be merged into adjoining encoded word(s).
+B<Warning:> this is a quick-and-dirty solution, intended for character
+sets which overlap ASCII.  B<It does not comply with the RFC-1522
+rules regarding the use of encoded words in message headers>.
+You may want to roll your own variant,
+using C<encode_mimeword()>, for your application.
+I<Thanks to Jan Kasprzak for reminding me about this problem.>
 
-B<Imcompatibility with MIME::Words>:
-L<MIME::Words> takes care of natural word separators (i.e. whitespaces)
-in the text to be encoded.  This module will encode whole text
-(if encoding needed) not mentioning whitespaces;
-encoded-words exceeding line length will be splitted based only on their
-lengths.
+B<Change by this module>:
+Encoded-words are concatenated then are splitted taking care of character
+boundaries of multibyte sequences.
 
 =cut
 
 sub encode_mimewords  {
     my $words = shift;
     my %params = @_;
-    my $charset = uc($params{'Charset'});
+    my $charset = $params{'Charset'};
     my $detect7bit = uc($params{'Detect7bit'} || "YES");
     my $encoding = uc($params{'Encoding'});
     my $header_name = $params{'Field'};
@@ -516,28 +424,25 @@ sub encode_mimewords  {
 	my ($s, $cset) = @$_;
 	my $enc;
 
-	next unless length($s);
-
 	# Unicode string should be encoded by given charset.
 	# Unsupported charset will be fallbacked to UTF-8.
-	if (is_utf8($s) or $s =~ $WIDECHAR) {
-	    unless (resolve_alias($cset)) {
+	if (Encode::is_utf8($s)) {
+	    unless (Encode::resolve_alias($cset)) {
 		if ($s !~ $UNSAFE) {
 		    $cset = "US-ASCII";
 		} else {
 		    $cset = "UTF-8";
 		}
 	    }
-	    $s = encode($cset, $s);
+	    $s = Encode::encode($cset, $s);
 	}
 
 	# Determine charset and encoding.
 	if ($encoding eq "A") {
 	    ($s, $cset, $enc) =
-		header_encode($s, $cset || $charset,
-			      Detect7bit => $detect7bit);
+		header_encode($s, $cset, Detect7bit => $detect7bit);
 	} else {
-	    $cset ||= ($charset || ($s !~ $UNSAFE)? "US-ASCII": "ISO-8859-1");
+	    $cset ||= ($s !~ $UNSAFE)? "US-ASCII": "ISO-8859-1";
 	    $enc = $encoding ||
 		(($s !~ $UNSAFE and $cset eq "US-ASCII")? undef: "Q");
 	}
@@ -549,11 +454,11 @@ sub encode_mimewords  {
 	if (scalar(@triplets)) {
 	    my ($last, $lastenc, $lastcset) = @{$triplets[-1]};
 	    if (uc($lastcset) eq uc($cset) and uc($lastenc) eq uc($enc) and
-		resolve_alias($cset)) {
+		Encode::resolve_alias($cset)) {
 		${$triplets[-1]}[0] .= $s;
 		next;
 	    } elsif (!$lastenc and $enc and $last !~ /[\t ]$/) {
-		if ($last =~ /^(.*)[\t ]([$PRINTABLE]+)$/s) {
+		if ($last =~ /^(.*)[\t ]([\x21-\x7E]+)$/s) {
 		    ${$triplets[-1]}[0] = $1." ";
 		    $s = $2.$s;
 		} elsif (uc($lastcset) eq "US-ASCII") {
@@ -563,7 +468,7 @@ sub encode_mimewords  {
 		    next;
 		}
 	    } elsif ($lastenc and !$enc and $s !~ /^[\t ]/) {
-		if ($s =~ /^([$PRINTABLE]+)[\t ](.*)$/s) {
+		if ($s =~ /^([\x21-\x7E]+)[\t ](.*)$/s) {
 		    ${$triplets[-1]}[0] .= $1;
 		    $s = " ".$2;
 		} elsif (uc($cset) eq "US-ASCII") {
@@ -575,7 +480,7 @@ sub encode_mimewords  {
 	push @triplets, [$s, $enc, $cset];
     }
 
-    # Split long ``words''.
+    # Split.
     my @splitted;
     my $restlen = $firstlinelen;
     my $lastlen = 0;
@@ -609,15 +514,15 @@ sub encode_mimewords  {
 	} else {
 	    $s = &encode_mimeword($str, $encoding, $charset);
 	}
+	my $spc = (scalar(@lines) and $lines[-1] =~ / $/)? '': ' ';
 
-	my $spc = (scalar(@lines) and $lines[-1] =~ /[\t ]$/)? '': ' ';
 	if (!scalar(@lines)) {
-	    $s =~ s/^[\r\n\t ]+//;
+	    $s =~ s/^\s+//;
 	    push @lines, $s;
 	} elsif (length($lines[-1]) + length($s) <= $linelen) {
 	    $lines[-1] .= $spc.$s;
 	} else {
-	    $s =~ s/^[\r\n\t ]+//;
+	    $s =~ s/^\s+//;
 	    push @lines, $s;
 	    $linelen = $MAXLINELEN - 1;
 	}
@@ -638,11 +543,10 @@ sub _split {
     my $charset = shift;
     my $restlen = shift;
 
-    if (!$charset or $charset eq '8BIT') {	# Undecodable.
-	$str =~ s/[\r\n]+[\t ]*|\x00/ /g;	# Eliminate hostile characters.
+    if (!$charset or $charset eq '8BIT') { # Undecodable.
 	return ([$str, undef, $charset]);
     }
-    unless (resolve_alias($charset)) {		# Unsupported charset.
+    unless (Encode::resolve_alias($charset)) { # Unsuppoted charset.
 	return ([$str, $encoding, $charset]);
     }
     if (!$encoding and $charset eq 'US-ASCII') {
@@ -656,7 +560,7 @@ sub _split {
 	    last;
 	}
 	$ustr = $str;
-	$ustr = decode($charset, $ustr);
+	$ustr = Encode::decode($charset, $ustr);
 	($first, $str) = &_clip_unsafe($ustr, $encoding, $charset, $restlen);
 	push @splitted, [$first, $encoding, $charset];
 	$restlen = $MAXLINELEN - 1;
@@ -665,67 +569,46 @@ sub _split {
 }
 
 # _split_ascii RAW, ROOM_OF_FIRST_LINE
-#     Private: used by encode_mimewords() to split an US-ASCII string into
+#     Private: used by encode_mimewords() to split an ASCII string into
 #     (encoded or non-encoded) words.
-#     Returns an array of arrayrefs [SUBSTRING, ENCODING, "US-ASCII"],
-#     where ENCODING is either undef or (if any unsafe sequences are
-#     included) "Q".
+#     Returns an array of arrayrefs [SUBSTRING, ENCODING, "US-ASCII"].
 sub _split_ascii {
     my $s = shift;
-    my $restlen = shift || $MAXLINELEN - 1;
+    my $firstlinelen = shift;
+    my $restlen = $firstlinelen;
 
     my @splitted;
     foreach my $line (split(/[\r\n]+/, $s)) {
 	$line =~ s/^[\t ]+//;
 
-	if (length($line) < $restlen and $line !~ /=\?|$UNSAFE/) {
-	    push @splitted, [$line, undef, "US-ASCII"];
+	if (length($line) < $restlen) {
+	    push @splitted, [$line, undef, 'US-ASCII'];
 	    $restlen = $MAXLINELEN - 1;
 	    next;
 	}
 
-        my ($spc, $enc);
+	my @words;
+        my $spc;
 	foreach my $word (split(/([\t ]+)/, $line)) {
-	    if ($word =~ /[\t ]/) {
+	    unless ($word =~ /[^\t ]/) {
 		$spc = $word;
 		next;
 	    }
-
-	    $enc = ($word =~ /=\?|$UNSAFE/)? "Q": undef;
-	    if (scalar(@splitted)) {
-		my ($last, $lastenc, $lastcset) = @{$splitted[-1]};
-		my ($elen, $cont, $appe);
-
-		# Concatenate adjacent words so that encoded-word and
-		# unencoded text will adjoin with separating whitespace.
-		if (!$lastenc and !$enc) {
-		    $elen = length($spc.$word);
-		    ($cont, $appe) = ($spc.$word, "");
-		} elsif (!$lastenc and $enc) {
-		    $elen = length($spc) +
-			encoded_header_len($word, "Q", "US-ASCII");
-		    ($cont, $appe) = ($spc, $word);
-		} elsif ($lastenc and !$enc) {
-		    $elen = length($spc.$word);
-		    ($cont, $appe) = ("", $spc.$word);
+	    if (scalar(@words)) {
+		if (length($spc.$word) < $restlen) {
+		    $words[-1] .= $spc.$word;
+		    $restlen -= length($spc.$word);
 		} else {
-		    $elen = encoded_header_len($spc.$word, "Q",
-					       "US-ASCII") - 15;
-		    ($cont, $appe) = ($spc.$word, "");
+		    push @words, $word;
+		    $restlen = $MAXLINELEN - length($word) - 1;
 		}
-
-		if ($elen <= $restlen) {
-		    ${$splitted[-1]}[0] .= $cont if length($cont);
-		    push @splitted, [$appe, $enc, "US-ASCII"] if length($appe);
-		    $restlen -= $elen;
-		    next;
-		}
-		$restlen = $MAXLINELEN - 1;
+	    } else {
+		push @words, $word;
+		$restlen -= length($word);
 	    }
-	    push @splitted, [$word, $enc, "US-ASCII"];
-	    $restlen -= ($enc?
-		encoded_header_len($word, "Q", "US-ASCII"):
-		length($word));
+	}
+	foreach (@words) {
+	    push @splitted, [$_, undef, 'US-ASCII'];
 	}
     }
     return @splitted;
@@ -742,10 +625,10 @@ sub _clip_unsafe {
     return ("", "") unless length($ustr);
 
     # Seek maximal division point.
-    my ($shorter, $longer) = (0, length($ustr));
+    my ($shorter, $longer) = (0, length($ustr)-1);
     while ($shorter < $longer) {
 	my $cur = int(($shorter + $longer + 1) / 2);
-	my $enc = encode($charset, substr($ustr, 0, $cur));
+	my $enc = Encode::encode($charset, substr($ustr, 0, $cur));
 	my $elen = encoded_header_len($enc, $encoding, $charset);
 	if ($elen <= $restlen) {
 	    $shorter = $cur;
@@ -762,15 +645,15 @@ sub _clip_unsafe {
 	eval {
 	    ($fenc, $renc) =
 		(substr($ustr, 0, $shorter), substr($ustr, $shorter));
-	    $fenc = encode($charset, $fenc, FB_CROAK());
-	    $renc = encode($charset, $renc, FB_CROAK());
+	    $fenc = &Encode::encode($charset, $fenc, Encode::FB_CROAK);
+	    $renc = &Encode::encode($charset, $renc, Encode::FB_CROAK);
 	};
 	last unless ($@);
 
-	$shorter++;
 	unless ($shorter < $max) { # Unencodable characters are included.
-	    return (encode($charset, $ustr), "");
+	    return (Encode::encode($charset, $ustr), "");
 	}
+	$shorter++;
     }
 
     if (length($fenc)) {
@@ -784,27 +667,15 @@ sub _clip_unsafe {
 
 =back
 
-=head1 VERSION
-
-Consult $VERSION variable.
-
-Development versions of this module may be found at
-L<http://hatuka.nezumi.nu/repos/MIME-EncWords/>.
-
-=head1 SEE ALSO
-
-L<MIME::Charset>,
-L<MIME::Tools>
-
 =head1 AUTHORS
 
 The original version of function decode_mimewords() is derived from
-L<MIME::Words> module that was written by:
+MIME::Words module that was written by:
     Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
     David F. Skoll (dfs@roaringpenguin.com) http://www.roaringpenguin.com
 
-Other stuff are rewritten or added by:
-    Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>.
+Other stuffs are rewritten or added by:
+    Hatuka*nezumi - IKEDA Soji (F<hatuka@nezumi.nu>).
 
 All rights reserved.  This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
